// (C) 2017-2020 by folkert van heusden, released under AGPL v3.0
#include <atomic>
#include <chrono>
#include <string>
#include <time.h>
#include <unistd.h>
#include <vector>
#include <sys/time.h>

#include "error.h"
#include "source.h"
#include "utils.h"
#include "picio.h"
#include "target.h"
#include "filter.h"
#include "log.h"
#include "exec.h"
#include "motion_trigger.h"
#include "motion_trigger_other_source.h"

motion_trigger_other_source::motion_trigger_other_source(const std::string & id, const std::string & descr, source *const hr_s, instance_t *const lr_motion_trigger, std::vector<target *> *const targets, const std::map<std::string, parameter *> & detection_parameters) : motion_trigger(id, descr, detection_parameters), hr_s(hr_s), lr_motion_trigger(lr_motion_trigger), targets(targets)
{
	ct = CT_MOTIONTRIGGER;

	for(auto mi : find_motion_triggers(lr_motion_trigger))
		static_cast<motion_trigger *>(mi)->register_notifier(this);
}

motion_trigger_other_source::~motion_trigger_other_source()
{
	for(auto mi : find_motion_triggers(lr_motion_trigger))
		static_cast<motion_trigger *>(mi)->unregister_notifier(this);

	stop();

	for(target *t : *targets)
		delete t;
	delete targets;
}

void motion_trigger_other_source::operator()()
{
	set_thread_name("motion_trigger_other_source");

	hr_s -> unregister_user();

	int duration = 0;
	bool recording = false;

	for(;!local_stop_flag;) {
		std::unique_lock<std::mutex> lk(m_event);

		using namespace std::chrono_literals;

		if (cv_event.wait_for(lk, 100ms) == std::cv_status::no_timeout) {
			lk.unlock();

			if (!recording) {
				log(id, LL_INFO, "starting");

				for(size_t i=0; i<targets -> size(); i++)
					targets -> at(i) -> start(nullptr, -1); // FIXME for all targets

				recording = true;
			}

			duration = 0;
		}

		if (recording) {
			duration++;

			if (duration > 123) {
				log(id, LL_INFO, "stopping");

				for(target *t : *targets)
					t -> stop();

				recording = false;
			}
		}
	}

	hr_s -> unregister_user();
}
